package logistics.material;

import logistics.quantities.FloatUnit;

/**
 * The LiterDiesel class represents a unit of measurement for diesel fuel.
 * It is a singleton class, meaning only one instance of the class can exist at a time.
 *
 * @author Nikolaus Köberlein
 * @version 1.0
 */
public class LiterDiesel implements FloatUnit {

    private static LiterDiesel INSTANCE;

    private LiterDiesel() {}

    /**
     * Get the singleton instance of the LiterDiesel class
     *
     * @return The singleton instance of the LiterDiesel class
     */
    public static LiterDiesel getINSTANCE() {
        if (INSTANCE == null) {
            INSTANCE = new LiterDiesel();
        }
        return INSTANCE;
    }

    /**
     * Get a string representation of the LiterDiesel object
     *
     * @return A string representation of the LiterDiesel object
     */
    @Override
    public String toString() {
        return "liters of diesel";
    }
}